/**********************************************************************-*-C-*-
 *
 * portlib.c - This file is part of `xradio'
 *
 * Copyright (C) 1996 Pat Thoyts (thoyts@bsa.bristol.ac.uk)
 *
 * This file provides all the routines used to send and receive data to and
 * from the ports. To work, this file MUST be compiled with -O2 and NOT with
 * -g and the resulting executable has to be suid root.
 *
 *****************************************************************************/

#include <unistd.h>
#include <asm/io.h>
#include "portlib.h"

/*
 * allow_io() and disallow_io() use the system call `ioperm' to enable use
 * of low numbered ports. These functions insist upon root access.
 *
 * We also look at /dev/console and see who owns it and whether it matches our
 * real user id (our euid is root) using access("/dev/console", R_OK);
 *
 * Locking? Nah? Maybe some idiot will run two copies - what happens?
 */
int
allow_io( unsigned long base, unsigned long num )
{
  int tmp;
  if ( geteuid() != 0
#if !defined(NO_CHECK_CONSOLE)
       || access("/dev/console", R_OK) != 0
#endif
       )
    return -1;
  tmp = ioperm(base, num, 1);
  return tmp;
}

int
disallow_io( unsigned long base, unsigned long num )
{
  int tmp;
  if ( geteuid() != 0
#if !defined(NO_CHECK_CONSOLE)
       || access("/dev/console", R_OK) != 0
#endif
       )
    return -1;

  tmp = ioperm(base, num, 0);
  return tmp;
}

/*
 * If we have port access, read or write 8 or 16 bit values from the ports
 * opened.
 */
unsigned int
read_byte ( unsigned int port )
{
  unsigned int value = 0;
  value = inb( port );
  return value;
}

unsigned int
read_word ( unsigned int port )
{
  unsigned int value = 0;
  value = (inb(port + 1)<<8);
  value &= inb(port);
  return value;    
}

void
write_byte ( unsigned int byte, unsigned int port )
{
  byte &= 0x00FF;
  outb( byte, port );
#ifdef DEBUG_PORTLIB
  fprintf(stderr, "PORTLIB: writing 8 bit value 0x%02X to 0x%03X\n",
	  byte, port);
#endif
}

void
write_word ( unsigned long word, unsigned int port )
{
  unsigned int hi = 0, lo = 0;
  hi = ((word & 0xFF00)>>8);
  lo = (word & 0x00FF);
  outw( lo, port );
#ifdef DEBUG_PORTLIB
  fprintf(stderr, "PORTLIB: writing 16 bit value 0x%04lx to 0x%03x\n",
	  word, port);
#endif
}

/*
 * Read data from ports from `base' to `base+num' and output on stream.
 * Basically for debugging purposes.
 */
void
print_ports ( FILE* stream, unsigned long base, unsigned long num )
{
  unsigned long cn = 0;

  fprintf(stream, "Ports:");
  for (cn = 0; cn < num; cn++) {
    fprintf(stream, " %04lX:%0X", base + cn, read_byte(base + cn));
  }
  fprintf(stream, "\n");
}

