/**********************************************************************-*-C-*-
 *
 * radio.c - This file is part of `xradio'
 *
 * Copyright (C) 1996 Pat Thoyts (thoyts@bsa.bristol.ac.uk)
 *
 *
 *****************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <getopt.h>
#include "reveal.h"

void usage ( void );

int
main( int argc, char** argv )
{
  float freq = 0;
  char c;
  int option_index = 0, ultrashort = 0;
  static const char* short_options = "hv10murs:f:";
  static struct option long_options[] = {
    { "help",     0, 0, 'h' },
    { "usage",    0, 0, 'h' },
    { "version",  0, 0, 'v' },
    { "on",       0, 0, '1' },
    { "off",      0, 0, '0' },
    { "louder",   0, 0, '+' },
    { "quieter",  0, 0, '-' },
    { "mute",     0, 0, 'm' },
    { "unmute",   0, 0, 'u' },
    { "set",      1, 0, 's' },
    { "frequency",1, 0, 'f' },
    { "read",     0, 0, 'r',},
    { 0,          0, 0,  0  }
  };

  if (argc < 2) {
    usage();
    exit (1);
  }

  ultrashort = 1;
  do {
    if (*(argv[ultrashort]+1) == '\0') {
      switch (*argv[ultrashort]) {
      case '-':
	++ultrashort;
	radio_quiet(0);
	break;
      case '+':
	++ultrashort;
	radio_loud(0);
	break;
      case '1':
	++ultrashort;
	radio_poweron(0);
	break;
      case '0':
	++ultrashort;
	radio_off();
	break;
      default:
	ultrashort = 0;
      }
    }
    else
      ultrashort = 0;
  } while (ultrashort && ultrashort < argc );

  while ((c = getopt_long(argc, argv, short_options,
			  long_options, &option_index)) != -1) {
    switch(c) {
    case 0:      /* if long option and flag is set then give up. */
      break;
    case 'h':
      usage();
      break;
    case '+':
      radio_loud(0);
      break;
    case '-':
      radio_quiet(0);
      break;
    case '1':
      radio_poweron(0);
      break;
    case '0':
    case 'm':
      radio_off();
      break;
    case 'u':
      radio_on();
      break;
      break;
    case 's':
    case 'f':
      freq = atof(optarg);
      radio_set(freq);
      break;
    case 'r':
      radio_read();
      break;
    default:
      break;
    }
  }

  return 0;
}

void
usage ( void )
{
  fprintf(stderr,
"usage: radio -[hv] [10+-] -[mu] -[fs] <frequency>
       --help       -h          : display usage information.
       --usage
       --version    -v          : print the program version number.
       --on          1          : switch the device on.
       --off         0          : turn the radio off.
       --louder      +          : turn up the volume.
       --quieter     -          : lower the volume.
       --mute       -m          : mute the radio.
       --unmute     -u          : stop muting the radio.
       --set <freq> -s <freq>   : tune radio to the given frequency    
       --frequency <freq>\n");
}



