/**********************************************************************-*-C-*-
 *
 * reveal.h - This file is part of `xradio'
 *
 * Copyright (C) 1996 Pat Thoyts (thoyts@bsa.bristol.ac.uk)
 *
 *
 *****************************************************************************/

#ifndef _REVEAL_H
#define _REVEAL_H

#ifndef RADIO_BASEIO
#  define RADIO_BASEIO 0x30C  /* the default io address for the card. */
#endif
#define RADIO_NUMIO 6         /* the number of ports reserved by the device */
#define RADIO_SLEEP 100000    /* the sleep period used between commands sent*/

#define RADIO_MAXHz 109.000   /* the maximum possible frequency */
#define RADIO_MINHz  87.000   /* the minimum possible frequency */
#ifndef RADIO_DEFHz
#  define RADIO_DEFHz 98.950  /* the default frequency to use */
#endif

typedef struct cmd_type {
  short value;
  short portoffset;
} RadioCommand;

/* These are the commands sent to the card. It looks like the port offset
 * is always 3 (0x30C + 0x3 = 0x30F) but the others out to be used for
 * something or why does the card grab 0x30C-0x310?
 * Oh well, an offset of 0xFF finishes the list.
 *
 *                   value, port offset.... */
#define RADIO_PWR   { {0x0088, 3}, {0x00C8, 3}, {0x00, 0xFF} } /*power on*/
#define RADIO_ON    { {0x00A0, 3}, {0x00C8, 3}, {0x00, 0xFF} } /*on from mute*/
#define RADIO_OFF   { {0x0000, 3},              {0x00, 0xFF} } /*off or mute*/
#define RADIO_INC   { {0x0088, 3}, {0x00C8, 3}, {0x00, 0xFF} } /*louder*/
#define RADIO_DEC   { {0x0048, 3}, {0x00C8, 3}, {0x00, 0xFF} } /*quieter*/
#define RADIO_HZ1   { {0x0005, 3}, {0x0005, 3}, \
                      {0x0007, 3}, {0x0007, 3}, {0x00, 0xFF} } /*freq set bit*/
#define RADIO_HZ0   { {0x0001, 3}, {0x0001, 3}, \
		      {0x0003, 3}, {0x0003, 3}, {0x00, 0xFF} } /* & clear bit*/
#define RADIO_HZE   { {0x0000, 3}, {0x00C8, 3}, {0x00, 0xFF} } /*freq end*/

/* Errors returned by these functions */
#define RADIO_E_NONE     0
#define RADIO_E_UNKNOWN  1
#define RADIO_E_RANGE    2
#define RADIO_E_PERM     3

/* Public Interface Functions */

int radio_poweron(float freq);
int radio_on( void );
int radio_off( void );
int radio_loud(int num);
int radio_quiet(int num);
int radio_set(float freq);
int radio_read(void);

#endif /* _REVEAL_H */

