/**********************************************************************-*-C-*-
 *
 * xradio.h - This file is part of `xradio'
 *
 * Copyright (C) 1996 Pat Thoyts (thoyts@bsa.bristol.ac.uk)
 *
 * This file provides two X interfaces to the radio card functions. Setting
 * the XRadio.simple resource to True will use a very basic interface, while
 * a lot more functionality is available by default.
 *
 *****************************************************************************/

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/Box.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/AsciiText.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/Repeater.h>
#include <X11/Xaw/Toggle.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include "reveal.h"
#ifdef HAVE_XPM
#  include <X11/xpm.h>
#  include "mute.xpm"
#  include "unmute.xpm"
#else
# include "mute.xbm"
# include "unmute.xbm"
#endif

typedef struct {
  float frequency;
  String name;
} RadioStation;
#define XpRStation (String)"Station"
#define XpCStation (String)"Station"

void muteCallback(Widget w, XtPointer clientData, XtPointer callData);
void louderCallback(Widget w, XtPointer clientData, XtPointer callData);
void quieterCallback(Widget w, XtPointer clientData, XtPointer callData);
void quitCallback(Widget w, XtPointer clientData, XtPointer callData);
void stationCallback(Widget w, XtPointer clientData, XtPointer callData);
void fineCallback(Widget w, XtPointer clientData, XtPointer callData);
void coarseCallback(Widget w, XtPointer clientData, XtPointer callData);
void volumeCallback(Widget w, XtPointer clientData, XtPointer callData);
void scanCallback(Widget w, XtPointer clientData, XtPointer callData);
void tuneRadio(Widget w, float step);

void setStartupStation(Widget shell);
void setRadioDisplay(Widget shell, RadioStation station);
void SetFrequency(Widget w, XEvent* event, String* params,
		   Cardinal* num_params);
static Boolean CvtStringToStation(Display* dpy, XrmValue* args, Cardinal* num,
				  XrmValue* fromVal, XrmValue* toVal,
				  XtPointer* cvt_data);
void createPlusMinusBox(Widget parent, const char* name,
			XtCallbackProc ourCallback);
int simple_interface(Widget shell);
int complex_interface(Widget shell);

static XtActionsRec app_actions[] = {
  { (String)"setFrequency", SetFrequency }
};

typedef struct {
  Boolean useBitmaps;
  Boolean simpleInterface;
  String muteLabel;
  String unmuteLabel;
  Pixmap muteBitmap;
  Pixmap unmuteBitmap;
  float coarseStep;
  float fineStep;
  RadioStation currentStation;
  RadioStation defstation;
  RadioStation stations[10];
} radioOptions_t, *radioOptions_p;
radioOptions_t radioOptions;

static XtResource resources[] = {
  { (String)"simpleInterface", (String)"SimpleInterface", XtRBoolean,
    sizeof(Boolean), XtOffset(radioOptions_p, simpleInterface),
    XtRImmediate, (XtPointer)FALSE },
  { (String)"useBitmaps", (String)"UseBitmaps", XtRBoolean, sizeof(Boolean),
    XtOffset(radioOptions_p, useBitmaps), XtRImmediate, (XtPointer)FALSE },
  { (String)"muteLabel", XtCLabel, XtRString, sizeof(String),
    XtOffset(radioOptions_p,muteLabel), XtRString, (XtPointer)"Mute" },
  { (String)"unmuteLabel", XtCLabel, XtRString, sizeof(String),
    XtOffset(radioOptions_p, unmuteLabel), XtRString, (XtPointer)"Unmute" },
  { (String)"muteBitmap", XtCPixmap, XtRBitmap, sizeof(Pixmap),
    XtOffset(radioOptions_p, muteBitmap), XtRImmediate,
    (XtPointer)XtUnspecifiedPixmap },
  { (String)"unmuteBitmap", XtCPixmap, XtRBitmap, sizeof(Pixmap),
    XtOffset(radioOptions_p, unmuteBitmap), XtRImmediate,
    (XtPointer)XtUnspecifiedPixmap },
  { (String)"coarseStep", (String)"TuneStep", XtRFloat, sizeof(float),
    XtOffset(radioOptions_p, coarseStep), XtRString, (XtPointer)"0.500" },
  { (String)"fineStep", (String)"TuneStep", XtRFloat, sizeof(float),
    XtOffset(radioOptions_p, fineStep), XtRString, (XtPointer)"0.025" },

  { (String)"station1", XpCStation, XpRStation, sizeof(RadioStation),
    XtOffset(radioOptions_p, stations[1]), XtRString, (XtPointer)None },
  { (String)"station2", XpCStation, XpRStation, sizeof(RadioStation),
    XtOffset(radioOptions_p, stations[2]), XtRString, (XtPointer)None },
  { (String)"station3", XpCStation, XpRStation, sizeof(RadioStation),
    XtOffset(radioOptions_p, stations[3]), XtRString, (XtPointer)None },
  { (String)"station4", XpCStation, XpRStation, sizeof(RadioStation),
    XtOffset(radioOptions_p, stations[4]), XtRString, (XtPointer)None },
  { (String)"station5", XpCStation, XpRStation, sizeof(RadioStation),
    XtOffset(radioOptions_p, stations[5]), XtRString, (XtPointer)None },
  { (String)"station6", XpCStation, XpRStation, sizeof(RadioStation),
    XtOffset(radioOptions_p, stations[6]), XtRString, (XtPointer)None },
  { (String)"station7", XpCStation, XpRStation, sizeof(RadioStation),
    XtOffset(radioOptions_p, stations[7]), XtRString, (XtPointer)None },
  { (String)"station8", XpCStation, XpRStation, sizeof(RadioStation),
    XtOffset(radioOptions_p, stations[8]), XtRString, (XtPointer)None },
  { (String)"station9", XpCStation, XpRStation, sizeof(RadioStation),
    XtOffset(radioOptions_p, stations[9]), XtRString, (XtPointer)None },
  { (String)"station10", XpCStation, XpRStation, sizeof(RadioStation),
    XtOffset(radioOptions_p, stations[10]), XtRString, (XtPointer)None },
  { (String)"defaultStation", XpCStation, XpRStation, sizeof(RadioStation),
    XtOffset(radioOptions_p, defstation), XtRString, (XtPointer)None },
};


int
main (int argc, char** argv)
{
  Widget shell;
  XtAppContext app;

  shell = XtAppInitialize(&app, "XRadio", NULL, 0,
			  &argc, argv, NULL, NULL, 0);
  XtAppAddActions(app, app_actions, XtNumber(app_actions));
  XtAppSetTypeConverter(app, XtRString, XpRStation, CvtStringToStation,
		    NULL, 0, XtCacheNone, NULL);
  XtGetApplicationResources(shell, &radioOptions, resources,
			    XtNumber(resources), NULL,0);

  if (radioOptions.muteBitmap == XtUnspecifiedPixmap) {
#ifdef HAVE_XPM
    XpmAttributes attributes;
    Pixmap mask;

    XtVaGetValues(shell, XtNdepth, &attributes.depth,
		  XtNcolormap, &attributes.colormap, NULL);
    attributes.visual = DefaultVisual(XtDisplay(shell),
				      DefaultScreen(XtDisplay(shell)));
    attributes.valuemask = XpmDepth | XpmColormap | XpmVisual;
    XpmCreatePixmapFromData(XtDisplay(shell),
			    RootWindowOfScreen(XtScreen(shell)),
			    unmute, &radioOptions.muteBitmap, &mask,
			    &attributes);
    if (mask) XFreePixmap(XtDisplay(shell), mask);
#else
    radioOptions.muteBitmap = 
      XCreateBitmapFromData(XtDisplay(shell),
			    RootWindowOfScreen(XtScreen(shell)),
			    unmute_bits, unmute_width, unmute_height);
#endif
  }

  if (radioOptions.unmuteBitmap == XtUnspecifiedPixmap) {
#ifdef HAVE_XPM
    XpmAttributes attributes;
    Pixmap mask;

    XtVaGetValues(shell, XtNdepth, &attributes.depth,
		  XtNcolormap, &attributes.colormap, NULL);
    attributes.visual = DefaultVisual(XtDisplay(shell),
				      DefaultScreen(XtDisplay(shell)));
    attributes.valuemask = XpmDepth | XpmColormap | XpmVisual;
    XpmCreatePixmapFromData(XtDisplay(shell),
			    RootWindowOfScreen(XtScreen(shell)),
			    mute, &radioOptions.unmuteBitmap, &mask,
			    &attributes);
    if (mask) XFreePixmap(XtDisplay(shell), mask);
#else
    radioOptions.unmuteBitmap = 
      XCreateBitmapFromData(XtDisplay(shell),
			    RootWindowOfScreen(XtScreen(shell)),
			    mute_bits, mute_width, mute_height);
#endif
  }

  if(radioOptions.simpleInterface)
    simple_interface(shell);
  else
    complex_interface(shell);
  
  XtRealizeWidget(shell);
  XtAppMainLoop(app);
  return 0;
}

int
simple_interface(Widget shell)
{
  Widget box, buttons, w, display;
  box = XtVaCreateManagedWidget( "simple", boxWidgetClass, shell,
				 XtNorientation, XtorientVertical,
				 NULL);
  
  display = XtVaCreateManagedWidget( "display", asciiTextWidgetClass, box,
				     XtNtype, XawAsciiString,
				     XtNeditType, XawtextEdit,
				     XtNuseStringInPlace, False,
				     NULL);

  buttons = XtVaCreateManagedWidget( "buttons", boxWidgetClass, box,
				     XtNorientation, XtorientHorizontal,
				     NULL);

  w = XtVaCreateManagedWidget( "mute", toggleWidgetClass, buttons,
			       NULL);
  XtAddCallback(w, XtNcallback, muteCallback, NULL);
  if (radioOptions.useBitmaps)
    XtVaSetValues(w, XtNbitmap, radioOptions.muteBitmap, NULL);
  else
    XtVaSetValues(w, XtNlabel, radioOptions.muteLabel, NULL);


  w = XtVaCreateManagedWidget( "quieter", commandWidgetClass, buttons,
			       NULL);
  XtAddCallback(w, XtNcallback, quieterCallback, NULL);
  
  w = XtVaCreateManagedWidget( "louder", commandWidgetClass, buttons,
			       NULL);
  XtAddCallback(w, XtNcallback, louderCallback, NULL);

  w = XtVaCreateManagedWidget( "quit", commandWidgetClass, buttons,
			       NULL);
  XtAddCallback(w, XtNcallback, quitCallback, NULL);

  return 0;
}

void
quitCallback(Widget w, XtPointer clientData, XtPointer callData)
{
  exit(0);
}

void
louderCallback(Widget w, XtPointer clientData, XtPointer callData)
{
  radio_loud(0);
}

void
quieterCallback(Widget w, XtPointer clientData, XtPointer callData)
{
  radio_quiet(0);
}

void
muteCallback(Widget w, XtPointer clientData, XtPointer callData)
{
  if ((long)callData == FALSE) { /* if toggle is unset */
    radio_on();
    if (radioOptions.useBitmaps)
      XtVaSetValues(w, XtNbitmap, radioOptions.muteBitmap, NULL);
    else
      XtVaSetValues(w, XtNlabel, radioOptions.muteLabel, NULL);
  }
  else {
    radio_off();
    if (radioOptions.useBitmaps)
      XtVaSetValues(w, XtNbitmap, radioOptions.unmuteBitmap, NULL);
    else
      XtVaSetValues(w, XtNlabel, radioOptions.unmuteLabel, NULL);
  }
}

void
SetFrequency(Widget w, XEvent* event, String* params, Cardinal* num_params)
{
  char* buffer;
  long cn;

  XtVaGetValues(w, XtNstring, &buffer, NULL);
  for(cn = 0 ; buffer[cn] != 0; cn++) {
    if ( !(isdigit(buffer[cn]) || buffer[cn] == '.') ) {
      XtWarning("The frequency value is invalid.\nMust be a number.");
      return;
    }
  }
  radio_set(atof(buffer));
  /* XawAsciiSourceFreeString(w);*/
}



/*******************************************************************
 * now the complex interface
 *******************************************************************/

int
complex_interface(Widget shell)
{
  Widget form, buttons, w, display, stations, stntop, stnbot;
  Cardinal cn;
  form = XtVaCreateManagedWidget( "complex", formWidgetClass, shell,
				 NULL);
  
  display = XtVaCreateManagedWidget( "display", labelWidgetClass, form,
				     NULL);
  w = XtVaCreateManagedWidget( "displayfreq", labelWidgetClass, form,
			       NULL);
  createPlusMinusBox(form, "scan", scanCallback);

  buttons = XtVaCreateManagedWidget( "buttons", boxWidgetClass, form,
				     XtNorientation, XtorientHorizontal,
				     NULL);

  stations = XtVaCreateManagedWidget( "stations", boxWidgetClass, form,
				     XtNorientation, XtorientVertical,
				     NULL);
  stntop = XtVaCreateManagedWidget( "top", boxWidgetClass, stations,
				     XtNorientation, XtorientHorizontal,
				    NULL);
  stnbot = XtVaCreateManagedWidget( "bottom", boxWidgetClass, stations,
				    XtNorientation, XtorientHorizontal,
				    NULL);
  {
    const char* row[] =
    { "station1", "station2", "station3", "station4", "station5" };
    for (cn = 0; cn < XtNumber(row); cn++) {
      w = XtVaCreateManagedWidget((String)row[cn], commandWidgetClass,
				  stntop, NULL);
      if (radioOptions.stations[cn+1].name != NULL &&
	  *(radioOptions.stations[cn+1].name) != 0)
	XtVaSetValues(w, XtNlabel, radioOptions.stations[cn+1].name, NULL);
      XtAddCallback(w, XtNcallback, stationCallback, NULL);
    }
  }
  
  {
    const char* row[] =
    { "station6", "station7", "station8", "station9", "station10" };
    for (cn = 0; cn < XtNumber(row); cn++) {
      w = XtVaCreateManagedWidget(row[cn],commandWidgetClass, stnbot, NULL);
      if (radioOptions.stations[cn+6].name != NULL &&
	  *(radioOptions.stations[cn+6].name) != 0)
	XtVaSetValues(w, XtNlabel, radioOptions.stations[cn+6].name, NULL);
      XtAddCallback(w, XtNcallback, stationCallback, NULL);
    }
  }
  
  w = XtVaCreateManagedWidget( "mute", toggleWidgetClass, buttons,
			       NULL);
  XtAddCallback(w, XtNcallback, muteCallback, NULL);
  if (radioOptions.useBitmaps)
    XtVaSetValues(w, XtNbitmap, radioOptions.muteBitmap, NULL);
  else
    XtVaSetValues(w, XtNlabel, radioOptions.muteLabel, NULL);

  createPlusMinusBox(buttons, "volume", volumeCallback);
  createPlusMinusBox(buttons, "fine", fineCallback);
  createPlusMinusBox(buttons, "coarse", coarseCallback);

  w = XtVaCreateManagedWidget( "quit", commandWidgetClass, buttons,
			       NULL);
  XtAddCallback(w, XtNcallback, quitCallback, NULL);

  setStartupStation(shell);
  return 0;
}

void
volumeCallback(Widget w, XtPointer clientData, XtPointer callData)
{
  String name;

  name = XtName(w);
  if (name == NULL)
    XtError("Oops. Failed to identify calling widget");

  if (strcmp(name, "minus") == 0)
    radio_quiet(0);
  else
    radio_loud(0);
}

void
fineCallback(Widget w, XtPointer clientData, XtPointer callData)
{
  String name;

  name = XtName(w);
  if (name == NULL)
    XtError("Oops. Failed to identify calling widget");

  if (strcmp(name, "minus") == 0)
    tuneRadio(w, -(radioOptions.fineStep));
  else
    tuneRadio(w, +(radioOptions.fineStep));
}

void
coarseCallback(Widget w, XtPointer clientData, XtPointer callData)
{
  String name;

  name = XtName(w);
  if (name == NULL)
    XtError("Oops. Failed to identify calling widget");

  if (strcmp(name, "minus") == 0)
    tuneRadio(w, -(radioOptions.coarseStep));
  else
    tuneRadio(w, +(radioOptions.coarseStep));
}

void
scanCallback(Widget w, XtPointer clientData, XtPointer callData)
{
  String name;

  name = XtName(w);
  if (name == NULL)
    XtError("Oops. Failed to identify calling widget");

  if (strcmp(name, "minus") == 0)
    XtWarning("Sorry, Not implemented yet.");
  else
    XtWarning("Sorry, Not implemented yet.");
}

void
tuneRadio(Widget w, float step)
{
  Cardinal r = 0;
  float newfreq = 0;
  RadioStation station;

  newfreq = radioOptions.currentStation.frequency + step;
  
  if (newfreq <= RADIO_MINHz || newfreq >= RADIO_MAXHz)
    newfreq = radioOptions.currentStation.frequency;

  station.name = (String)"Tuning..";
  station.frequency = newfreq;

  if (station.frequency != 0)
    r = radio_set(station.frequency);

  if (0 == 0) {
    Widget shell = w;
    do {
      shell = XtParent(shell);
    } while(!(XtIsTopLevelShell(shell)));
    setRadioDisplay(shell, station);
  }
}

void
setStartupStation(Widget shell)
{
  Widget w = NULL;
  RadioStation* station = NULL;
  Cardinal cn = 0, r = 0, match = 0, len = 0;

  station = &(radioOptions.defstation);

  /* If there is both a freq and name, just use these.*/
  if (*(station->name) != 0 && station->frequency != 0) {
    match = -1;
    r = radio_set(station->frequency);
    if (r == r) 
      setRadioDisplay(shell, *station);
  }
  else if (*(station->name) != 0) {
    len = strlen(station->name);
    for (cn = 1; cn <= XtNumber(radioOptions.stations) && match == 0; cn++) {
      if (strncmp(station->name, radioOptions.stations[cn].name, len) == 0) {
	match = -1;
	r = radio_set(radioOptions.stations[cn].frequency);
	if (r == r) 
	  setRadioDisplay(shell, radioOptions.stations[cn]);
      }
    }
  }
  else if (station->frequency != 0) { /* try and find default by freq */
    for (cn = 1; cn <= XtNumber(radioOptions.stations) && match == 0; cn++) {
      if (station->frequency == radioOptions.stations[cn].frequency) {
	match = -1;
	r = radio_set(radioOptions.stations[cn].frequency);
	if (r == r) 
	  setRadioDisplay(shell, radioOptions.stations[cn]);
      }
    }
  }
  else { /*try for programmed station 1*/
    w = XtNameToWidget(shell, "*station1");
    if (w == NULL)
      XtWarning("Oops. Can't find station1 widget.");
    stationCallback(w, NULL, NULL);
  }
}

void
stationCallback(Widget w, XtPointer clientData, XtPointer callData)
{
  String name = NULL;
  Cardinal stn = 0, r = 0;
  RadioStation* station = NULL;

  name = XtName(w);
  while (!isdigit(*name)) ++name;
  stn = strtol(name, NULL, 0);
  station = &(radioOptions.stations[stn]);
  /* 
   *fprintf(stderr, "%.3f %s\n", station->frequency, station->name);
   */

  if (station->frequency != 0)
    r = radio_set(station->frequency);

  if (0 == 0) {
    Widget shell = w;
    do {
      shell = XtParent(shell);
    } while(!(XtIsTopLevelShell(shell)));

    setRadioDisplay(shell, *station);
  }
}

void
setRadioDisplay(Widget shell, RadioStation station)
{
    Widget y = NULL, z = NULL;
    String name = NULL;

    radioOptions.currentStation.name = station.name;
    radioOptions.currentStation.frequency = station.frequency;

    y = XtNameToWidget(shell, "complex*display");
    z = XtNameToWidget(shell, "complex*displayfreq");

    if (y != NULL && XtIsWidget(y)) {
      XtVaSetValues(y, XtNlabel, station.name, NULL);
    }
    else
      XtWarning("Cannot find the display widget");

    name = (String)XtMalloc(sizeof(char) * 30);
    sprintf(name, "%.3f", station.frequency);
    if (z != NULL && XtIsWidget(z))
      XtVaSetValues(z, XtNlabel, name, NULL);
    else
      XtWarning("Cannot find the displayfreq widget");
    XtFree((XtPointer)name);
    
    return;
}

/* Resource converter to convert radio station entried into a frequency and 
 * a name component.
 */
static Boolean
CvtStringToStation(Display* dpy, XrmValue* args, Cardinal* num,
		   XrmValue* fromVal, XrmValue* toVal, XtPointer* cvt_data)
{
  RadioStation* station = NULL;
  String name = NULL;

  if (fromVal == NULL || fromVal->addr == NULL)
    return FALSE;
  
  station = (RadioStation*)XtMalloc(sizeof(RadioStation));
  station->frequency = strtod((String)fromVal->addr, &name);
  while((*name == ' ' || *name == '\t') && *name != 0)
    ++name;
  station->name = name;

  if (toVal->addr != NULL) {
    if (toVal->size != sizeof(RadioStation)) {
      XtError("String-to-RadioStation converter - memory allocated is insufficient.\n");
      return FALSE;
    }
    memcpy( (*toVal).addr, station, (*toVal).size);
    XtFree((XtPointer)station);
  }
  else {
    (*toVal).size = sizeof(RadioStation);
    (*toVal).addr = (XtPointer)station;
  }
  return TRUE;
}

void
createPlusMinusBox(Widget parent, const char* name,
		   XtCallbackProc ourCallback)
{
  Widget box, label, plus, minus;
  Cardinal height = 0;

  box = XtVaCreateManagedWidget( (String)name, boxWidgetClass, parent,
				 XtNorientation, XtorientHorizontal,
				 XtNborderWidth, 0,
				 XtNhSpace, 0, XtNvSpace, 0,
				 NULL);
  minus=XtVaCreateManagedWidget("minus",repeaterWidgetClass, box, NULL);
  XtAddCallback(minus, XtNcallback, ourCallback, NULL);
  label=XtVaCreateManagedWidget("label", commandWidgetClass, box,
	      XtNtranslations,
	      XtParseTranslationTable("#override <BtnDown>:  "),
				NULL);
  plus= XtVaCreateManagedWidget("plus", repeaterWidgetClass, box, NULL);
  XtAddCallback(plus, XtNcallback, ourCallback, NULL);
  XtVaGetValues(label, XtNheight, &height, NULL);
  XtVaSetValues(minus, XtNheight, height, NULL);
  XtVaSetValues(plus, XtNheight, height, NULL);
}
